<?php

namespace FiloBlu\Flow\Model\Channel\In;

class Anagrafica extends AbstractModel {

    /**
     * @var Monolog\Logger
     */
    protected $_logger;
    protected $_storesByLocale = array();
    protected $_channel_config;
    protected $_resourceconnection;
    protected $_helper;
    protected $_special_action_properties;

    public function __construct(
        \FiloBlu\Flow\Helper\LoggerProvider $loggerProvider,
        \FiloBlu\Flow\Model\Channel\Config $channelConfig,
        \Magento\Framework\App\ResourceConnection $resourceConnection,
        \FiloBlu\Flow\Helper\Data $helper
    ) {
        parent::__construct($loggerProvider, $channelConfig);
        $this->_logger = $loggerProvider->getLogger();
        $this->_channel_config = $channelConfig;
        $this->_resourceconnection = $resourceConnection;
        $this->_helper = $helper;

        $this->_storeByLocale = $helper->getAllStoresCodeByLocale();

    }

    public function load($channelname) {
        $this->init($channelname);
        $this->_special_action_properties = $this->_helper->getSpecialActionsPriorities();
        return $this;
    }

    // If $execution_string start with "_" then it is a Flow Action type
    public function isFlowAction($execution_string){
        $execution_string = trim($execution_string);
        if ($execution_string[0] == "_"){
            return true;
        }
        return false;
    }

    public function generateEavsTemplateBind($data,$action=null, $fieldMap, $meta_file)
    {
        $templates = array();

        $meta_ref_id = (isset($this->_special_action_properties[$data['a']]) && isset($this->_special_action_properties[$data['a']]['priority'])? $this->_special_action_properties[$data['a']]['priority'] : 1);
        $executed_on = (isset($this->_special_action_properties[$data['a']]) && isset($this->_special_action_properties[$data['a']]['executed_on'])? $this->_special_action_properties[$data['a']]['executed_on'] : null);
        // Non special actions should have 0
        if(!$this->isFlowAction($data['a'])){
            $meta_ref_id = 0;
            $executed_on = "";
        }
        // Minimum for special actions should be 1
        if($this->isFlowAction($data['a']) && $meta_ref_id == 0){
            $meta_ref_id = 1;
        }

        $template_bind = array(
            'e' => $data['e'],
            'a' => $data['a'],
            'v' => trim($data['v']),
            'l' => $data['l'],
            'meta_file' => $meta_file,
            'meta_ref_id' => $meta_ref_id,
            'executed_on' => $executed_on,
            'meta_processed' => 0,
            'meta_insert_time' => date("Y-m-d H:i:s", time())
        );

        // Aggiungo le storeview (se presenti nel mapping)
        $storeViews = array('s' => $data['s']);
        if (isset($fieldMap["stores"]) && count($fieldMap["stores"]) > 0 && $fieldMap["stores"]) {
            $storeViews = $fieldMap["stores"];
        } else if (isset($fieldMap["locale"]) && count($fieldMap["locale"]) > 0 && $fieldMap["locale"]) {
            $storeViews = $this->_storeByLocale[$fieldMap["locale"]];
        }
        foreach ($storeViews as $key => $store) {
            $template_bind['s'] = $store;
            $templates[] = $template_bind;
        }

        return $templates;
    }

    public function getTemplateBindFields($data)
    {
        $binds = array();
        $meta_file = $this->getFile()->getId();

        $parser = $this->getParser();
        $check_mapping_count = $parser->check_mapping_count;

        $is_eavs_direct_file = false;
        if (isset($data['e']) && isset($data['a']) && isset($data['v']) && isset($data['s'])) {
            $is_eavs_direct_file = true;
        }

        // Anagrafica with no mapping count check, then this CSV is a direct EAVS CSV
        // Like this:
        /*
        array(4) {
        ["e"]=> "9008336JJB000000XX"
        ["a"]=> "barcode"
        ["v"]=> "1232131321321"
        ["s"]=> ""
        */
        if ($is_eavs_direct_file) {

            $fieldMap = array();
            if($this->map){
                $fieldMap = $this->map->get($data['a']);
            }

            // You can't choose a filedMap To on a EAVS direct file (but mapping it's still readable), forcing it!
            $fieldMap["to"] = $data['a'];
            $value = trim($data['v']);
            $action = null;
            if (!isset($fieldMap["skip"]) && ($value != '' || isset($fieldMap["allowNull"]))) {
                $binds = array_merge($binds, $this->generateEavsTemplateBind($data,$action, $fieldMap, $meta_file));
            }
            if (isset($fieldMap["actions"])) {
                foreach ($fieldMap["actions"] as $action) {
                    // Queste action funzionano diversamente
                    if (in_array($action, array('_translate_attribute', '_set_default_locale'))) {
                        $binds = array_merge($binds, $this->generateEavsTemplateBind($data['a'], $action, $fieldMap, $meta_file));
                    } else {
                        // Action normale basata sullo SKU
                        $binds = array_merge($binds, $this->generateEavsTemplateBind($data['e'], $action, $fieldMap, $meta_file));
                    }
                }
            }

            // This is a direct EAVS file, stop here!
            return $binds;
        }

        // Anagrafica with mapping count check, then this CSV is a column field CSV
        // Like this:
        /*
        array(4) {
        ["sku"]=> "9008336JJB000000XX"
        ["barcode"]=> "135454861436483643"
        ["name"]=> "Shirt"
        ["color"]=> "Blue"
        [...]
        */
        // Per
        foreach ($data as $field => $value) {

            $fieldMap = array();
            if($this->map){
                $fieldMap = $this->map->get($field);
            }

            $value = addslashes($value);
            $to_field = "";

            if(!$check_mapping_count){
                // You can't choose a filedMap To on not $check_mapping_count file so, forcing it!
                $fieldMap["to"] = $field;
            }

            if (!isset($fieldMap["skip"]) && (trim($value) != '' || isset($fieldMap["allowNull"]))) {
                if (isset($fieldMap["to"]) && $fieldMap["to"]) {
                    $to_field = $fieldMap["to"];
                    // Qua ho coperto tutti gli attributi base senza special action con un "To" nel mapping
                    $binds = array_merge($binds, $this->generateEavsTemplateBind($data, $fieldMap, $meta_file));
                }

                // Qua vado a coprire le varie special action
                if ($to_field && isset($fieldMap["actions"])) {
                    foreach ($fieldMap["actions"] as $action) {
                        // Queste action funzionano diversamente
                        if (in_array($action, array('_translate_attribute', '_set_default_locale'))) {
                            $binds = array_merge($binds, $this->generateEavsTemplateBind($data['a'], $fieldMap, $meta_file));
                            // Action normale basata sullo SKU
                        } else {
                            $binds = array_merge($binds, $this->generateEavsTemplateBind($data['sku'], $action, $fieldMap, $meta_file));
                        }
                    }
                }
            }
        }

        return $binds;
    }

    /*
     * DEPRECATED
     */
    public function getSimplefields($data, $meta_file)
    {

        $binds = ["simple"=>[],"actions"=>[]];

        foreach ($data as $field => $value) {
            $fieldMap = $this->map->get($field);
            $value = addslashes($value);

            if (!isset($fieldMap["skip"]) && (trim($value) != '' || isset($fieldMap["allowNull"]))) {
                $to_field = "";
                if (isset($fieldMap["to"]) && $fieldMap["to"]) {
                    $to_field = $fieldMap["to"];
                    if (!isset($fieldMap['actions']) || !in_array('_translate_attribute', $fieldMap['actions'])) {
                        //devo inserire una riga per ogni attributo
                        $template_bind = array(
                            'e' => $data['sku'],
                            'a' => $to_field,
                            'v' => trim($value),
                            'meta_file' => $meta_file,
                            's' => '',
                            'l' => $data['l'],
                            'meta_ref_id' => 0,
                            'meta_processed' => 0,
                            'meta_insert_time' => date("Y-m-d H:i:s", time())
                        );

                        try {

                            $storeViews = array('s'=>'');

                            if (isset($fieldMap["stores"]) && count($fieldMap["stores"]) > 0 && $fieldMap["stores"]) {
                                $storeViews = $fieldMap["stores"];
                            } else if (isset($fieldMap["locale"]) && count($fieldMap["locale"]) > 0 && $fieldMap["locale"]) {
                                $storeViews = $this->_storeByLocale[$fieldMap["locale"]];
                            }

                            foreach ($storeViews as $store) {
                                $template_bind['s'] = $store;
                                $binds["simple"][] = $template_bind;
                            }

                        } catch (\Exception $e) {
                            // $writeConnection->rollback();
                            throw new \Exception($e->getMessage());
                        }
                    }
                }

            }
            //le azioni le conservo per inserirle alla fine
            if (isset($fieldMap["actions"])) {
                foreach ($fieldMap["actions"] as $action) {
                    $a = array("value" => $value, "stores" => array());
                    if (isset($to_field) && ($action == '_translate_attribute' || $action == '_set_default_locale')) {
                        $a['e'] = $to_field;
                    }
                    if (isset($fieldMap["stores"]) && count($fieldMap["stores"]) > 0 && $fieldMap["stores"]) {
                        $a['stores'] = $fieldMap["stores"];
                    } else if (isset($fieldMap["locale"]) && count($fieldMap["locale"]) > 0 && $fieldMap["locale"]) {
                        $storeViews = $this->_storeByLocale[$fieldMap["locale"]];
                        foreach ($storeViews as $store) {
                            $a['stores'][] = $store;
                        }
                    }


                    $binds["actions"][$action][] = $a;
                }
            }
        }

        return $binds;

    }


    /*
     * DEPRECATED
     */
    public function getActionfields($data,$actions,$meta_file) {

        $actions_parserd = [];

        foreach($actions as $a => $values){
            foreach($values as $v) {
                if(in_array($a, array('_add_related', '_add_upsell', '_add_crosssell'))){
                    //$padre = explode('#',$data['cod_configurabile']);
                    $bind = array(
                        'e' => $data['sku'],
                        'a' => $a,
                        'v' => $v['value'],
                        'meta_file' => $meta_file,
                        's' => '',
                        'meta_ref_id' => (isset($v['e']) ? 1 : 2),
                        'meta_processed' => 0,
                        'meta_insert_time' => date("Y-m-d H:i:s", time())
                    );
                }
                else{
                    $bind = array(
                        'e' => (isset($v['e']) ? $v['e'] : $data['sku']),
                        'a' => $a,
                        'v' => $v['value'],
                        'meta_file' => $meta_file,
                        's' => '',
                        'meta_ref_id' => (isset($v['e']) ? 1 : 2),
                        'meta_processed' => 0,
                        'meta_insert_time' => date("Y-m-d H:i:s", time())
                    );
                }
                if (count($v['stores']) > 0) {
                    $storeViews = $v['stores'];
                    foreach ($storeViews as $store) {
                        $bind['s'] = $store;
                        $actions_parserd[] = $bind;
                    }
                } else {
                    $actions_parserd[] = $bind;
                }
            }
        }


        return $actions_parserd;

    }

    public function insertData($data){

        $flow = $this->getFile()->getData('flow');
        $model =  \Magento\Framework\App\ObjectManager::getInstance()->create('\FiloBlu\Flow\Model' . "\\" . $flow);

        if (!$model)
            return false;

        $resource = $this->_resourceconnection;

        $writeConnection = $resource->getConnection(\Magento\Framework\App\ResourceConnection::DEFAULT_CONNECTION);
        $tableName = $resource->getTableName('flow_from_eavs');

        $meta_file = $this->getFile()->getId();
        $actions = array();

        $query = "INSERT INTO {$tableName} (e,a,v,s,l,meta_file,meta_ref_id,executed_on,meta_processed,meta_insert_time) VALUES (:e,:a,:v,:s,:l,:meta_file,:meta_ref_id,:executed_on,:meta_processed,:meta_insert_time)";

        // Deprecated (untouched) functions
        //$binds_raw = $this->getSimplefields($data,$meta_file);
        //$actions_raw =$this->getActionfields($data,$binds_raw["actions"],$meta_file);
        //$binds = array_merge($binds_raw["simple"],$actions_raw);

        $binds = $this->getTemplateBindFields($data);

        try {

            $writeConnection->insertOnDuplicate($tableName, $binds);

        } catch (\Exception $e) {
            $this->_logger->info($e->getMessage());
        }


    }

    public function receiveFile($file){

        //$this->_logger->info(__METHOD__);

        if (!$this->_connector)
            throw new \Exception('no connector joined to channel');

        $file_channel_config = \Magento\Framework\App\ObjectManager::getInstance()->create('\FiloBlu\Flow\Model\Channel\Config')->load($file->getChannel())->getConfigData();

        // se il flag è true, quando viene caricato un file anagrafica, automaticamente viene duplicato per il flusso ite
        // passo anche il nome del channel ite con la configurazione corretta - se il campo è vuoto, il default sarà "ite"
        $ite = $file_channel_config['config']['copy_to_ite'];
        $ite_channel_name = ($file_channel_config['config']['ite_channel_name']) ? $file_channel_config['config']['ite_channel_name'] : 'ite';

        if($ite){
            $received = $this->_connector->receiveResource($file->getName(), 'ite_'.date('Ymd_Hmi'));
        } else {
            $received = $this->_connector->receiveResource($file->getName());
        }
        //Se ho copiato il fine creo una riga per il flusso
        if(is_array($received)){

            $copy = $received['copy'];
            $received = $received['basefile'];
            $model = \Magento\Framework\App\ObjectManager::getInstance()->create('FiloBlu\Flow\Model\Inboundflow');;
            $model->setName($copy);
            $model->setFlow('From\Product');
            $model->setType('file');
            $model->setChannel($ite_channel_name);
            $model->setRetry(0);
            $model->setStatus('received');
            $model->setPriority('normal');
            try{
                $model->save();
            }catch (\Exception $e){
                $this->_logger->info($e->getMessage());
            }
        }
        elseif ($received === false) {
            throw new \Exception('no remote files found');
        }
        return $received;

    }


    protected function checkDuplicates($tableName,$bind){

        // $this->_logger->info(__METHOD__);

        $resource = \Magento\Framework\App\ObjectManager::getInstance()->create('\Magento\Framework\App\ResourceConnection');
        $readConnection = $resource->getConnection(\Magento\Framework\App\ResourceConnection::DEFAULT_CONNECTION);

        $query = "SELECT count(*) FROM {$tableName} WHERE e = '{$bind["e"]}' AND a = '{$bind["a"]}' AND v = '{$bind["v"]}' AND meta_file = '{$bind["meta_file"]}' AND s = '{$bind["s"]}'";
        $result = $readConnection->fetchOne($query);
        if($result > 0){
            return true;
        }
        return false;
    }


}
