<?php
namespace FiloBlu\Flow\Model\From;


abstract class AbstractFrom extends \Magento\Framework\Model\AbstractModel {

    const PRODUCT_NOT_FOUND_META_STATUS = -2;
    protected $_error_message;

    abstract public function process();

    public function setProcessed() {
        $now = date('Y-m-d H:i:s');
        $this->setMetaProcessed(true)
            ->setMetaProcessTime($now);
        return $this;
    }

    protected function error($blocking, $msg) {

        if ($blocking) {
            throw new \Exception($msg);
        } else {
            $this->setErrorMessage($this->_error_message."\r\n".$msg);
            return false;
        }
    }

    public function setErrorMessage($msg) {
        $this->_error_message = $msg;
    }

    public function getErrorMessage() {
        return $this->_error_message;
    }

    protected function preprocessFileRows($query,$file) {
        return $query;
    }

    protected function postprocessFileRows($query) {
        return true;
    }

    public function processFileRows($file) {

        $connection = $this->getResource()->getConnection();

        $have_executed_on_column = $connection->tableColumnExists($this->getResource()->getMainTable(), 'executed_on');

        // All query to execute by order
        $all_query = array();

        // If I have the 'executed on' column then I'm inside the EAVS
        if($have_executed_on_column){
            // 1st
            // All the EAVS actions (meta_ref_id = 0)
            // All the EAVS special actions executed on 'product' (executed_on = 'product')
            $rows = $this->getCollection()
                ->addFieldToFilter('meta_file',$file->getId())
                ->addFieldToFilter('meta_processed',0)
                ->addFieldToFilter(
                    array('meta_ref_id', 'executed_on'),
                    array(
                        array('eq'=>'0'),
                        array('eq'=>'product')
                    )
                );
            $rows->getSelect()->order('e DESC');
            $rows->getSelect()->order('meta_ref_id ASC');
            $all_query[] = $rows;

            // 2nd
            // All the EAVS special actions executed on 'attribute' (executed_on = 'attribute')
            $rows = $this->getCollection()
                ->addFieldToFilter('meta_file',$file->getId())
                ->addFieldToFilter('meta_processed',0)
                // Greater than 0 are only special actions
                ->addFieldToFilter('meta_ref_id',array('gt'=>'0'))
                ->addFieldToFilter('executed_on','attribute');
            $rows->getSelect()->order('e DESC');
            $rows->getSelect()->order('meta_ref_id ASC');
            $all_query[] = $rows;

            // 3rd
            // All the EAVS special actions remaining
            $rows = $this->getCollection()
                ->addFieldToFilter('meta_file',$file->getId())
                ->addFieldToFilter('meta_processed',0)
                // Greater than 0 are only special actions
                ->addFieldToFilter('meta_ref_id',array('gt'=>'0'));
            $rows->getSelect()->order('e DESC');
            $rows->getSelect()->order('meta_ref_id ASC');
            $all_query[] = $rows;

        // Fallback to the old default
        } else {
            $rows = $this->getCollection()
                ->addFieldToFilter('meta_file',$file->getId())
                ->addFieldToFilter('meta_processed',0)
                ->setOrder("meta_ref_id","ASC");
            $all_query[] = $rows;
        }


        // Running execution in order
        foreach ($all_query as $rows) {
            $query = $connection->query((String)$rows->getSelect());
            $query = $this->preprocessFileRows($query,$file);
            foreach($query->fetchAll() as $row){
                try{
                    $this->setData($row);
                    $this->process();
                    $connection->query("Update " . $this->getResource()->getMainTable() . " set meta_processed = 1, meta_process_time = '".date('Y-m-d H:i:s')."' where meta_id = ".$row["meta_id"]);
                }catch(\Exception $e){
                    //$connection->rollback();
                    $message = 'processing item '.$row["meta_id"].': '.$e->getMessage();
                    throw new \Exception($message);
                }
            }
        }

        $this->postprocessFileRows($query);
        return $file;

    }

    public function clean($file) {

        if (!$file->getId())
            return false;

        $connection = $this->getResource()->getConnection();

        try {

            $sql = "delete from " . $this->getResource()->getMainTable() ." where meta_file = " . $file->getId();
            $connection->query($sql);

        } catch (\Exception $e) {

            $message = 'processing item '.$file->getName().': '.$e->getMessage();
            throw new \Exception($message);

        }

    }

}